'use strict';

require('../globals');

var filter = module.exports = {};

var BINARY = {
  '>':  true,
  '>=': true,
  '=':  true,
  '!=': true,
  '<':  true,
  '<=': true
};

filter.addFilters = function(rawTable, encoding) {
  var filters = encoding.filter();  // apply filters to raw data before aggregation

  if (!rawTable.transform)
    rawTable.transform = [];

  // add custom filters
  for (var i=0, l=filters.length; i<l; i++) {
    var filter = filters[i];

    var condition = '';
    var operator = filter.operator;
    var operands = filter.operands;

    var d = 'd.' + (encoding._vega2 ? '' : 'data.');

    if (BINARY[operator]) {
      // expects a field and a value
      if (operator === '=') {
        operator = '==';
      }

      var op1 = operands[0];
      var op2 = operands[1];
      condition = d + op1 + operator + op2;
    } else if (operator === 'notNull') {
      // expects a number of fields
      for (var j=0; j<operands.length; j++) {
        condition += d + operands[j] + '!==null';
        if (j < operands.length - 1) {
          condition += ' && ';
        }
      }
    } else {
      console.warn('Unsupported operator: ', operator);
    }

    rawTable.transform.push({
      type: 'filter',
      test: condition
    });
  }

  return rawTable;
};

// remove less than 0 values if we use log function
filter.filterLessThanZero = function(dataTable, encoding) {
  encoding.forEach(function(field, encType) {
    if (encoding.scale(encType).type === 'log') {
      dataTable.transform.push({
        type: 'filter',
        test: 'd.' + encoding.fieldRef(encType) + '>0'
      });
    }
  });
};
